const gridElement = document.getElementById('grid');
const board = ['', '', '', '', '', '', '', '', ''];
let currentPlayer = 'X';
let playerTurn = true; // Flag to track whose turn it is
const resultText = document.getElementById('result'); // Assuming you have a <p id="result"> in your HTML

function createBoard() {
  for (let i = 0; i < 9; i++) {
    const cell = document.createElement('div');
    cell.className = 'cell';
    cell.addEventListener('click', () => handleCellClick(i));
    gridElement.appendChild(cell);
  }
  resultText.textContent = '';
}

function handleCellClick(index) {
  if (playerTurn && !board[index]) {
    board[index] = currentPlayer;
    render();
    let result = checkWinningResult();
    if (result) {
      if (result === 'draw') {
        resultText.textContent = "It's a draw! (Click to Restart)";
      } else {
        resultText.textContent = `${result} wins! (Click to Restart)`;
      }
      // Make the grid clickable to reset
      gridElement.removeEventListener('click', handleCellClick);
      setTimeout(() => {
        gridElement.addEventListener('click', resetGame);
      }, 1000);
    } else {
      playerTurn = false; // AI's turn now
      currentPlayer = currentPlayer === 'X' ? 'O' : 'X'; // Change player
      setTimeout(AIMove, 500); // AI makes a move after a delay
    }
  }
}

// Get the AI move
function AIMove() {
  const move = bestMove();
  if (move) {
    board[move.index] = currentPlayer;
    render();
    let result = checkWinningResult();
    if (result) {
      if (result === 'draw') {
        resultText.textContent = "It's a draw! (Click to Restart)";
      } else {
        resultText.textContent = `${result} wins! (Click to Restart)`;
      }
      // Make the grid clickable to reset
      gridElement.removeEventListener('click', handleCellClick);
      setTimeout(() => {
        gridElement.addEventListener('click', resetGame);
      }, 1000);
    } else {
      playerTurn = true; // Player's turn now
      currentPlayer = currentPlayer === 'X' ? 'O' : 'X'; // Change player
    }
  }
}

// Calculate best move for AI
const scores = {
  X: -1, // Opponent
  O: 1, // AI
  draw: 0,
};

const bestMove = () => {
  let bestScore = -Infinity;
  let move;

  // Iterate over all available spots on the board
  for (let i = 0; i < 3; i++) {
    for (let j = 0; j < 3; j++) {
      const index = i * 3 + j; // Convert 2D coordinates to 1D index
      // Is the spot available?
      if (board[index] === '') {
        board[index] = 'O'; // Assume AI is 'O'
        let score = minimax(board, 0, false);
        board[index] = ''; // Reset the board

        // Update best move if the score is better
        if (score > bestScore) {
          bestScore = score;
          move = { index: index }; // Save the index of the best move
        }
      }
    }
  }

  return move;
};

// Minimax algorithm
const minimax = (board, depth, isMaximizing) => {
  let result = checkWinningResult();
  if (result !== null) {
    return scores[result]; // Return the score based on the game outcome
  }

  // If it's the AI's turn (maximizing player)
  if (isMaximizing) {
    let bestScore = -Infinity;
    for (let i = 0; i < 3; i++) {
      for (let j = 0; j < 3; j++) {
        const index = i * 3 + j;
        if (board[index] === '') {
          board[index] = 'O'; // Make the move for AI
          let score = minimax(board, depth + 1, false); // Recursively call minimax for opponent's turn
          board[index] = ''; // Reset the board
          bestScore = Math.max(score, bestScore); // Keep the best score for AI
        }
      }
    }
    return bestScore;
  } else {
    // If it's the opponent's turn (minimizing player)
    let bestScore = Infinity;
    for (let i = 0; i < 3; i++) {
      for (let j = 0; j < 3; j++) {
        const index = i * 3 + j;
        if (board[index] === '') {
          board[index] = 'X'; // Make the move for opponent
          let score = minimax(board, depth + 1, true); // Recursively call minimax for AI's turn
          board[index] = ''; // Reset the board
          bestScore = Math.min(score, bestScore); // Keep the best score for opponent
        }
      }
    }
    return bestScore;
  }
};

// Function to check the winner
function checkWinningResult() {
  const winPatterns = [
    [0, 1, 2],
    [3, 4, 5],
    [6, 7, 8], // Horizontals
    [0, 3, 6],
    [1, 4, 7],
    [2, 5, 8], // Verticals
    [0, 4, 8],
    [2, 4, 6], // Diagonals
  ];

  for (const pattern of winPatterns) {
    const [a, b, c] = pattern;
    if (board[a] && board[a] === board[b] && board[a] === board[c]) {
      return board[a]; // Return the winner ('X' or 'O')
    }
  }

  // Check for draw - if all spaces are filled and no winner
  if (board.every((space) => space !== '')) {
    return 'draw';
  }

  return null; // No winner yet
}

function render() {
  const cells = document.querySelectorAll('.cell');
  for (let i = 0; i < cells.length; i++) {
    cells[i].textContent = board[i];
  }
}

function resetGame() {
  board.fill('');
  currentPlayer = 'X';
  playerTurn = true; // Reset the turn to player
  resultText.textContent = '';
  render();
  // Re-enable click events for the grid after reset
  gridElement.removeEventListener('click', resetGame);
  gridElement.addEventListener('click', handleCellClick);
}

// Initialize the game
createBoard();
